@@ -6,6 +6,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -27,8 +35,6 @@
 #include <unistd.h>
 
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-authentication-manager.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <gst/gst.h>
 
 #include <dbus/dbus.h>
@@ -52,14 +58,28 @@ typedef struct {
 	gboolean external;
 } ServiceData;
 
+static gboolean
+append_error (DBusMessageIter *iter,
+	      gint error_type,
+	      const char *message)
+{
+	if (!dbus_message_iter_append_basic (iter, DBUS_TYPE_UINT32, &error_type) ||
+	    !dbus_message_iter_append_basic (iter, DBUS_TYPE_STRING, &message)) {
+		rb_debug ("couldn't append error data");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static DBusHandlerResult
-_send_error (DBusConnection *connection,
-	     DBusMessage *request,
-	     gboolean include_flag,
-	     gint error_type,
-	     const char *message)
+send_error (DBusConnection *connection,
+	    DBusMessage *request,
+	    gint error_type,
+	    const char *message)
 {
 	DBusMessage *reply = dbus_message_new_method_return (request);
+	DBusMessageIter iter;
 
 	if (!message) {
 		message = "";
@@ -68,19 +88,8 @@ _send_error (DBusConnection *connection,
 		rb_debug ("attempting to return error: %s", message);
 	}
 
-	if (include_flag) {
-		gboolean ok = FALSE;
-		if (!dbus_message_append_args (reply, DBUS_TYPE_BOOLEAN, &ok, DBUS_TYPE_INVALID)) {
-			rb_debug ("couldn't append error flag");
-			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-		}
-	}
-
-	if (!dbus_message_append_args (reply,
-				       DBUS_TYPE_UINT32, &error_type,
-				       DBUS_TYPE_STRING, &message,
-				       DBUS_TYPE_INVALID)) {
-		rb_debug ("couldn't append error data");
+	dbus_message_iter_init_append (reply, &iter);
+	if (append_error (&iter, error_type, message) == FALSE) {
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
@@ -98,33 +107,26 @@ rb_metadata_dbus_load (DBusConnection *connection,
 	DBusMessageIter iter;
 	DBusMessage *reply;
 	GError *error = NULL;
-	gboolean ok = TRUE;
+	gboolean ok;
 	const char *mimetype = NULL;
+	char **missing_plugins = NULL;
+	char **plugin_descriptions = NULL;
+	gboolean has_audio;
+	gboolean has_video;
+	gboolean has_other_data;
 
 	if (!dbus_message_iter_init (message, &iter)) {
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
 	}
 
 	if (!rb_metadata_dbus_get_string (&iter, &uri)) {
-		/* make translatable? */
-		return _send_error (connection, message, TRUE,
-				    RB_METADATA_ERROR_INTERNAL,
-				    "Unable to read URI from request");
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
 	rb_debug ("loading metadata from %s", uri);
-
 	rb_metadata_load (svc->metadata, uri, &error);
+	rb_debug ("metadata load finished (type %s)", rb_metadata_get_mime (svc->metadata));
 	g_free (uri);
-	if (error != NULL) {
-		DBusHandlerResult r;
-		rb_debug ("metadata error: %s", error->message);
-
-		r = _send_error (connection, message, TRUE, error->code, error->message);
-		g_clear_error (&error);
-		return r;
-	}
-	rb_debug ("metadata load finished; mimetype = %s", rb_metadata_get_mime (svc->metadata));
 
 	/* construct reply */
 	reply = dbus_message_new_method_return (message);
@@ -132,21 +134,52 @@ rb_metadata_dbus_load (DBusConnection *connection,
 		rb_debug ("out of memory creating return message");
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
-
-	mimetype = rb_metadata_get_mime (svc->metadata);
+	
 	dbus_message_iter_init_append (reply, &iter);
 
-	if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &ok) ||
+	rb_metadata_get_missing_plugins (svc->metadata, &missing_plugins, &plugin_descriptions);
+	if (!rb_metadata_dbus_add_strv (&iter, missing_plugins)) {
+		rb_debug ("out of memory adding data to return message");
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+	if (!rb_metadata_dbus_add_strv (&iter, plugin_descriptions)) {
+		rb_debug ("out of memory adding data to return message");
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+
+	mimetype = rb_metadata_get_mime (svc->metadata);
+	if (mimetype == NULL) {
+		mimetype = "";
+	}
+	has_audio = rb_metadata_has_audio (svc->metadata);
+	has_video = rb_metadata_has_video (svc->metadata);
+	has_other_data = rb_metadata_has_other_data (svc->metadata);
+
+	if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &has_audio) ||
+	    !dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &has_video) ||
+	    !dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &has_other_data) ||
 	    !dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &mimetype)) {
 		rb_debug ("out of memory adding data to return message");
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
+	ok = (error == NULL);
+	if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &ok)) {
+		rb_debug ("out of memory adding error flag to return message");
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+
+	if (error != NULL) {
+		rb_debug ("metadata error: %s", error->message);
+		if (append_error (&iter, error->code, error->message) == FALSE) {
+			rb_debug ("out of memory adding error details to return message");
+			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+		}
+	}
+
 	if (!rb_metadata_dbus_add_to_message (svc->metadata, &iter)) {
-		/* make translatable? */
-		return _send_error (connection, message, TRUE,
-				    RB_METADATA_ERROR_INTERNAL,
-				    "Unable to add metadata to return message");
+		rb_debug ("unable to add metadata to return message");
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
 
 	if (!dbus_connection_send (connection, reply, NULL)) {
@@ -155,33 +188,17 @@ rb_metadata_dbus_load (DBusConnection *connection,
 	}
 
 	dbus_message_unref (reply);
-
 	return DBUS_HANDLER_RESULT_HANDLED;
 }
 
 static DBusHandlerResult
-rb_metadata_dbus_can_save (DBusConnection *connection,
-			   DBusMessage *message,
-			   ServiceData *svc)
+rb_metadata_dbus_get_saveable_types (DBusConnection *connection,
+				     DBusMessage *message,
+				     ServiceData *svc)
 {
-	char *mimetype;
 	DBusMessageIter iter;
 	DBusMessage *reply;
-	gboolean can_save;
-
-	if (!dbus_message_iter_init (message, &iter)) {
-		return DBUS_HANDLER_RESULT_NEED_MEMORY;
-	}
-
-	if (!rb_metadata_dbus_get_string (&iter, &mimetype)) {
-		/* make translatable? */
-		return _send_error (connection, message, TRUE,
-				    RB_METADATA_ERROR_INTERNAL,
-				    "Unable to read MIME type from request");
-	}
-
-	can_save = rb_metadata_can_save (svc->metadata, mimetype);
-	g_free (mimetype);
+	char **saveable_types;
 
 	/* construct reply */
 	reply = dbus_message_new_method_return (message);
@@ -191,10 +208,15 @@ rb_metadata_dbus_can_save (DBusConnection *connection,
 	}
 
 	dbus_message_iter_init_append (reply, &iter);
-	if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &can_save)) {
-		rb_debug ("out of memory adding data to return message");
+
+	saveable_types = rb_metadata_get_saveable_types (svc->metadata);
+
+	if (!rb_metadata_dbus_add_strv (&iter, saveable_types)) {
+		rb_debug ("out of memory adding saveable types to return message");
+		g_strfreev (saveable_types);
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
+	g_strfreev (saveable_types);
 
 	if (!dbus_connection_send (connection, reply, NULL)) {
 		rb_debug ("failed to send return message");
@@ -219,36 +241,44 @@ rb_metadata_dbus_save (DBusConnection *connection,
 		       DBusMessage *message,
 		       ServiceData *svc)
 {
+	char *uri;
 	DBusMessageIter iter;
 	DBusMessage *reply;
 	GHashTable *data;
 	GError *error = NULL;
 
-	/* get metadata from message */
+	/* get URI and metadata from message */
 	if (!dbus_message_iter_init (message, &iter)) {
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
 	}
+	if (!rb_metadata_dbus_get_string (&iter, &uri)) {
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
+
 	data = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify)rb_value_free);
 	if (!rb_metadata_dbus_read_from_message (svc->metadata,
 						 data,
 						 &iter)) {
 		/* make translatable? */
-		return _send_error (connection, message, FALSE,
-				    RB_METADATA_ERROR_INTERNAL,
-				    "Unable to read metadata from message");
+		g_free (uri);
+		return send_error (connection, message,
+				   RB_METADATA_ERROR_INTERNAL,
+				   "Unable to read metadata from message");
 	}
 
 	/* pass to real metadata instance, and save it */
+	rb_metadata_reset (svc->metadata);
 	g_hash_table_foreach_remove (data, (GHRFunc) _set_metadata, svc->metadata);
 	g_hash_table_destroy (data);
 
-	rb_metadata_save (svc->metadata, &error);
+	rb_metadata_save (svc->metadata, uri, &error);
+	g_free (uri);
 
 	if (error) {
 		DBusHandlerResult r;
 		rb_debug ("metadata error: %s", error->message);
 
-		r = _send_error (connection, message, FALSE, error->code, error->message);
+		r = send_error (connection, message, error->code, error->message);
 		g_clear_error (&error);
 		return r;
 	}
@@ -319,20 +349,23 @@ static DBusHandlerResult
 _handle_message (DBusConnection *connection, DBusMessage *message, void *data)
 {
 	ServiceData *svc = (ServiceData *)data;
-	rb_debug ("handling metadata service message");
+	DBusHandlerResult result;
+	rb_debug ("handling metadata service message: %s", dbus_message_get_member (message));
+
+	if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "load")) {
+		result = rb_metadata_dbus_load (connection, message, svc);
+	} else if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "getSaveableTypes")) {
+		result = rb_metadata_dbus_get_saveable_types (connection, message, svc);
+	} else if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "save")) {
+		result = rb_metadata_dbus_save (connection, message, svc);
+	} else if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "ping")) {
+		result = rb_metadata_dbus_ping (connection, message, svc);
+	} else {
+		result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
 
 	svc->last_active = time (NULL);
-	if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "load")) {
-		return rb_metadata_dbus_load (connection, message, svc);
-	} else if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "canSave")) {
-		return rb_metadata_dbus_can_save (connection, message, svc);
-	} else if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "save")) {
-		return rb_metadata_dbus_save (connection, message, svc);
-	} else if (dbus_message_is_method_call (message, RB_METADATA_DBUS_INTERFACE, "ping")) {
-		return rb_metadata_dbus_ping (connection, message, svc);
-	} else {
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-	}
+	return result;
 }
 
 static void
@@ -367,15 +400,21 @@ _new_connection (DBusServer *server,
 }
 
 static int
-test_can_save (const char *mimetype)
+test_saveable_types ()
 {
 	RBMetaData *md;
-	gboolean can_save;
+	char **saveable;
+	int i;
 
 	md = rb_metadata_new ();
-	can_save = rb_metadata_can_save (md, mimetype);
-	g_print ("%s save %s\n", can_save ? "Can" : "Can't", mimetype);
-	g_object_unref (G_OBJECT (md));
+	saveable = rb_metadata_get_saveable_types (md);
+	g_object_unref (md);
+
+	for (i = 0; saveable[i] != NULL; i++) {
+		g_print ("%s\n", saveable[i]);
+	}
+	g_strfreev (saveable);
+
 	return 0;
 }
 
@@ -385,17 +424,13 @@ test_load (const char *uri)
 	RBMetaData *md;
 	GError *error = NULL;
 	int rv = 0;
+	char **missing_plugins;
+	char **plugin_descriptions;
 
 	md = rb_metadata_new ();
 	rb_metadata_load (md, uri, &error);
 	if (error) {
-		if (error->code == RB_METADATA_ERROR_NOT_AUDIO_IGNORE) {
-			g_print ("%s is not an audio stream (ignored)\n", uri);
-		} else if (error->code == RB_METADATA_ERROR_NOT_AUDIO) {
-			g_print ("%s is not an audio stream\n", uri);
-		} else {
-			g_print ("Error loading metadata from %s: %s\n", uri, error->message);
-		}
+		g_print ("Error loading metadata from %s: %s\n", uri, error->message);
 		g_clear_error (&error);
 		rv = -1;
 	} else {
@@ -415,6 +450,21 @@ test_load (const char *uri)
 			}
 		}
 	}
+
+	g_print ("has audio: %d\n", rb_metadata_has_audio (md));
+	g_print ("has video: %d\n", rb_metadata_has_video (md));
+	g_print ("has other: %d\n", rb_metadata_has_other_data (md));
+
+	if (rb_metadata_get_missing_plugins (md, &missing_plugins, &plugin_descriptions)) {
+		int i = 0;
+		g_print ("missing plugins:\n");
+		while (missing_plugins[i] != NULL) {
+			g_print ("\t%s (%s)\n", missing_plugins[i], plugin_descriptions[i]);
+			i++;
+		}
+		g_strfreev (missing_plugins);
+	}
+
 	g_object_unref (G_OBJECT (md));
 	return rv;
 }
@@ -433,12 +483,8 @@ main (int argc, char **argv)
 	textdomain (GETTEXT_PACKAGE);
 #endif
 	g_type_init ();
-	gnome_vfs_init ();
-	gnome_authentication_manager_init ();
-#ifdef HAVE_GSTREAMER
 	gst_init (NULL, NULL);
 	g_set_prgname ("rhythmbox-metadata");
-#endif
 
 	if (argv[1] != NULL && strcmp(argv[1], "--debug") == 0) {
 		argv++;
@@ -454,8 +500,8 @@ main (int argc, char **argv)
 	if (argv[1] != NULL && strcmp(argv[1], "--load") == 0) {
 		return test_load (argv[2]);
 	}
-	if (argv[1] != NULL && strcmp(argv[1], "--can-save") == 0) {
-		return test_can_save (argv[2]);
+	if (argv[1] != NULL && strcmp(argv[1], "--saveable-types") == 0) {
+		return test_saveable_types ();
 	}
 
 	if (argv[1] != NULL && strcmp (argv[1], "--external") == 0) {
@@ -499,23 +545,21 @@ main (int argc, char **argv)
 				       g_main_loop_get_context (svc.loop));
 
 	if (!svc.external)
-		g_timeout_add (ATTENTION_SPAN * 500, (GSourceFunc) electromagnetic_shotgun, &svc);
+		g_timeout_add_seconds (ATTENTION_SPAN / 2, (GSourceFunc) electromagnetic_shotgun, &svc);
 
 	g_main_loop_run (svc.loop);
 
 	if (svc.connection) {
-#ifdef WITH_OLD_DBUS
-		dbus_connection_disconnect (svc.connection);
-#else
 		dbus_connection_close (svc.connection);
-#endif
 		dbus_connection_unref (svc.connection);
 	}
 
+	g_object_unref (svc.metadata);
+	g_main_loop_unref (svc.loop);
+
 	dbus_server_disconnect (svc.server);
 	dbus_server_unref (svc.server);
-#ifdef HAVE_GSTREAMER_0_10
 	gst_deinit ();
-#endif
+
 	return 0;
 }
