@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Abstraction of libgpod Itdb_ItunesDB object
- *
  *  Copyright (C) 2007 Christophe Fergeau  <teuf@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -8,7 +6,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -124,9 +122,12 @@ static void
 rb_itdb_save (RbIpodDb *ipod_db, GError **error)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+	GError *err = NULL;
 
-	itdb_write (priv->itdb, error);
-	if ((error != NULL) && (*error != NULL)) {
+	rb_debug ("Writing iPod database to disk");
+	if (itdb_write (priv->itdb, &err) == FALSE) {
+		g_warning ("Could not write database to iPod: %s", err->message);
+		g_propagate_error (error, err);
 		return;
 	}
 	if (priv->needs_shuffle_db) {
@@ -255,7 +256,10 @@ rb_ipod_free_delayed_action (RbIpodDelayedAction *action)
 		g_object_unref (action->thumbnail_data.pixbuf);
 		break;
 	case RB_IPOD_ACTION_ADD_TRACK:
-		itdb_track_free (action->track);
+		if (action->track != NULL) {
+			g_warning ("non NULL Itdb_Track, shouldn't happen");
+			itdb_track_free (action->track);
+		}
 		break;
 	case RB_IPOD_ACTION_ADD_PLAYLIST:
 		/* Do nothing */
@@ -605,7 +609,7 @@ rb_ipod_db_queue_remove_track (RbIpodDb *ipod_db,
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 	
 	g_assert (priv->read_only);
-	rb_debug ("Queueing move to trash action since the iPod database is currently read-only");
+	rb_debug ("Queueing track remove action since the iPod database is currently read-only");
 	action = g_new0 (RbIpodDelayedAction, 1);
 	action->type = RB_IPOD_ACTION_REMOVE_TRACK;
 	action->track = track;
@@ -739,36 +743,29 @@ rb_ipod_db_queue_set_thumbnail (RbIpodDb *ipod_db,
 	g_queue_push_tail (priv->delayed_actions, action);
 }
 
-static gchar *
-rb_ipod_db_get_volume_path (GnomeVFSVolume *volume)
-{
-	gchar *path;
-	gchar *uri;
-
-	uri = gnome_vfs_volume_get_activation_uri (volume);
-	path = g_filename_from_uri (uri, NULL, NULL);
-	g_assert (path != NULL);
-	g_free (uri);
-
-	return path;
-}
-
 static gboolean
-rb_ipod_db_load (RbIpodDb *ipod_db, GnomeVFSVolume *volume)
+rb_ipod_db_load (RbIpodDb *ipod_db, GMount *mount)
 {
+	GFile *mount_root;
 	char *mount_path;
 	const Itdb_IpodInfo *info;
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
-	mount_path = rb_ipod_db_get_volume_path (volume);
+	mount_root = g_mount_get_root (mount);
+	if (mount_root == NULL) {
+		return FALSE;
+	}
+	mount_path = g_file_get_path (mount_root);
+	g_object_unref (mount_root);
+
  	priv->itdb = itdb_parse (mount_path, NULL);
 	g_free (mount_path);
 
         if (priv->itdb == NULL) {
-            return FALSE;
+		return FALSE;
         }
 
-	info = itdb_device_get_ipod_info(priv->itdb->device);
+	info = itdb_device_get_ipod_info (priv->itdb->device);
 	if (info->ipod_generation == ITDB_IPOD_GENERATION_UNKNOWN ||
 	    info->ipod_generation == ITDB_IPOD_GENERATION_SHUFFLE_1 ||
 	    info->ipod_generation == ITDB_IPOD_GENERATION_SHUFFLE_2 ||
@@ -781,26 +778,25 @@ rb_ipod_db_load (RbIpodDb *ipod_db, GnomeVFSVolume *volume)
         return TRUE;
 }
 
-RbIpodDb *
-rb_ipod_db_new (GnomeVFSVolume *volume)
+RbIpodDb *rb_ipod_db_new (GMount *mount)
 {
 	RbIpodDb *db;
         gboolean success;
 
-	g_return_val_if_fail (volume != NULL, NULL);
+	g_return_val_if_fail (mount != NULL, NULL);
 
 	db = g_object_new (RB_TYPE_IPOD_DB, NULL);
 	if (db == NULL) {
 		return NULL;
 	}
 
-	success = rb_ipod_db_load (db, volume);
+	success = rb_ipod_db_load (db, mount);
 
-        if (success == FALSE) {
-            return NULL;
-        } else {
-	return db;
-}
+	if (success == FALSE) {
+		return NULL;
+	} else {
+		return db;
+	}
 }
 
 
@@ -831,7 +827,6 @@ saving_thread (RbIpodDb *ipod_db)
 
 	g_assert (priv->read_only);
 
-	rb_debug ("Writing iPod database to disk");
 	rb_itdb_save (ipod_db, NULL);
 	priv->save_idle_id = g_idle_add ((GSourceFunc)ipod_db_saved_idle_cb, 
 					 ipod_db);
@@ -865,9 +860,9 @@ rb_ipod_db_save_async (RbIpodDb *ipod_db)
 
 	if (priv->save_timeout_id == 0) {
 		rb_debug ("Scheduling iPod database save in 15 seconds");
-		priv->save_timeout_id = g_timeout_add (15000, 
-						       (GSourceFunc)save_timeout_cb,
-						       ipod_db);
+		priv->save_timeout_id = g_timeout_add_seconds (15, 
+							       (GSourceFunc)save_timeout_cb,
+							       ipod_db);
 	} else {
 		rb_debug ("Database save already scheduled");
 	}
@@ -878,7 +873,16 @@ rb_ipod_db_get_playlists (RbIpodDb *ipod_db)
 {
 	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
 
-	return priv->itdb->playlists;
+	return g_list_copy (priv->itdb->playlists);
+}
+
+Itdb_Playlist *
+rb_ipod_db_get_playlist_by_name (RbIpodDb *ipod_db,
+				 gchar *name)
+{
+	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+
+	return itdb_playlist_by_name (priv->itdb, name);
 }
 
 GList *
@@ -908,3 +912,10 @@ rb_ipod_db_get_device (RbIpodDb *ipod_db)
 	return priv->itdb->device;
 }
 
+guint32
+rb_ipod_db_get_database_version (RbIpodDb *ipod_db)
+{
+	RbIpodDbPrivate *priv = IPOD_DB_GET_PRIVATE (ipod_db);
+
+	return priv->itdb->version;
+}
