@@ -27,7 +27,7 @@ import JamendoConfigureDialog
 
 import os
 import gobject
-import gtk.glade
+import gtk
 import gnome, gconf
 import xml
 import gzip
@@ -95,6 +95,9 @@ class JamendoSource(rb.BrowserSource):
 	def do_impl_get_paned_key (self):
 		return "/apps/rhythmbox/plugins/jamendo/paned_position"
 
+	def do_impl_can_delete (self):
+		return False
+
 	def do_impl_pack_paned (self, paned):
 		self.__paned_box = gtk.VBox(False, 5)
 		self.pack_start(self.__paned_box)
@@ -120,7 +123,7 @@ class JamendoSource(rb.BrowserSource):
 			return (_("Loading Jamendo catalog"), None, progress)
 		else:
 			qm = self.get_property("query-model")
-			return (qm.compute_status_normal("%d song", "%d songs"), None, 0.0)
+			return (qm.compute_status_normal("%d song", "%d songs"), None, 2.0)
 
 	def do_impl_activate(self):
 		if not self.__activated:
@@ -132,7 +135,7 @@ class JamendoSource(rb.BrowserSource):
 			self.__show_loading_screen (True)
 
 			# start our catalogue updates
-			self.__update_id = gobject.timeout_add(6 * 60 * 60 * 1000, self.__update_catalogue)
+			self.__update_id = gobject.timeout_add_seconds(6 * 60 * 60, self.__update_catalogue)
 			self.__update_catalogue()
 
 			sort_key = gconf.client_get_default().get_string(JamendoConfigureDialog.gconf_keys['sorting'])
@@ -254,9 +257,11 @@ class JamendoSource(rb.BrowserSource):
 
 	def __show_loading_screen(self, show):
 		if self.__info_screen is None:
-			# load the glade stuff
-			gladexml = gtk.glade.XML(self.__plugin.find_file("jamendo-loading.glade"), root="jamendo_loading_scrolledwindow")
-			self.__info_screen = gladexml.get_widget("jamendo_loading_scrolledwindow")
+			# load the builder stuff
+			builder = gtk.Builder()
+			builder.add_from_file(self.__plugin.find_file("jamendo-loading.ui"))
+
+			self.__info_screen = builder.get_object("jamendo_loading_scrolledwindow")
 			self.pack_start(self.__info_screen)
 			self.get_entry_view().set_no_show_all (True)
 			self.__info_screen.set_no_show_all (True)
@@ -302,8 +307,8 @@ class JamendoSource(rb.BrowserSource):
 			gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, emsg).run()
 			return
 
-		rb.show_uri(result)
-	
+		gtk.show_uri(self.props.shell.props.window.get_screen(), result, gtk.gdk.CURRENT_TIME)
+
 	# Donate to Artist
 	def launch_donate (self):
 		tracks = self.get_entry_view().get_selected_entries()
@@ -325,7 +330,7 @@ class JamendoSource(rb.BrowserSource):
 			emsg = _("Error looking up artist %s on jamendo.com") % (artist)
 			gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, emsg).run()
 			return
-		rb.show_uri(result + "donate/")
+		gtk.show_uri(self.props.shell.props.window.get_screen(), result + "donate/", gtk.gdk.CURRENT_TIME)
 
 	def playing_entry_changed (self, entry):
 		if not self.__db or not entry:
