@@ -120,11 +120,10 @@ class MagnatuneSource(rb.BrowserSource):
 			return (_("Downloading Magnatune Album(s)"), None, progress)
 		else:
 			qm = self.get_property("query-model")
-			return (qm.compute_status_normal("%d song", "%d songs"), None, 0.0)
+			return (qm.compute_status_normal("%d song", "%d songs"), None, 2.0)
 
 	def do_impl_get_ui_actions(self):
-		return ["MagnatunePurchaseAlbum",
-			"MagnatuneDownloadAlbum",
+		return ["MagnatuneDownloadAlbum",
 			"MagnatuneArtistInfo",
 			"MagnatuneCancelDownload"]
 
@@ -201,7 +200,7 @@ class MagnatuneSource(rb.BrowserSource):
 				gtk.show_uri(screen, url, gtk.gdk.CURRENT_TIME)
 				urls.add(url)
 
-	def purchase_album(self):
+	def purchase_redirect(self):
 		screen = self.props.shell.props.window.get_screen()
 		tracks = self.get_entry_view().get_selected_entries()
 		urls = set([])
@@ -214,6 +213,11 @@ class MagnatuneSource(rb.BrowserSource):
 				urls.add(url)
 
 	def download_album(self):
+		if self.__client.get_string(self.__plugin.gconf_keys['account_type']) != 'download':
+			# The user doesn't have a download account, so redirect them to the purchase page.
+			self.purchase_redirect()
+			return
+
 		try:
 			library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0] # Just use the first library location
 		except IndexError, e:
@@ -569,13 +573,13 @@ class MagnatuneSource(rb.BrowserSource):
 
 	def __move_data_files(self):
 		# create cache and data directories
-		magnatune_in_progress_dir = magnatune_in_progress_dir.get_path()
-		magnatune_cache_dir = magnatune_cache_dir.get_path()
+		magnatune_in_progress_path = magnatune_in_progress_dir.get_path()
+		magnatune_cache_path = magnatune_cache_dir.get_path()
 
 		# (we know they don't already exist, and we know the parent dirs do)
-		os.mkdir(magnatune_in_progress_dir, 0700)
-		if os.path.exists(magnatune_cache_dir) is False:
-			os.mkdir(magnatune_cache_dir, 0700)
+		os.mkdir(magnatune_in_progress_path, 0700)
+		if os.path.exists(magnatune_cache_path) is False:
+			os.mkdir(magnatune_cache_path, 0700)
 
 		# move song info to cache dir
 		old_magnatune_dir = os.path.join(rb.dot_dir(), 'magnatune')
@@ -595,7 +599,7 @@ class MagnatuneSource(rb.BrowserSource):
 		for f in otherfiles:
 			print "moving file %s to new in-progress dir" % f
 			os.rename(os.path.join(old_magnatune_dir, f),
-				  os.path.join(magnatune_in_progress_dir, f))
+				  os.path.join(magnatune_in_progress_path, f))
 
 
 gobject.type_register(MagnatuneSource)
