@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Headfile of feed_podcast feed properties dialog
  *
  *  Copyright (C) 2005 Renato Araujo Oliveira Filho <renato.filho@indt.org>
  *
@@ -35,15 +33,18 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <glib.h>
+/* GStreamer happens to have some language name functions */
+#include <gst/gst.h>
+#include <gst/tag/tag.h>
 
 #include "rb-feed-podcast-properties-dialog.h"
 #include "rb-file-helpers.h"
-#include "rb-glade-helpers.h"
+#include "rb-builder-helpers.h"
 #include "rb-dialog.h"
 #include "rb-cut-and-paste-code.h"
 #include "rhythmdb.h"
+#include "rb-debug.h"
 
 static void rb_feed_podcast_properties_dialog_class_init (RBFeedPodcastPropertiesDialogClass *klass);
 static void rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog);
@@ -104,7 +105,8 @@ rb_feed_podcast_properties_dialog_class_init (RBFeedPodcastPropertiesDialogClass
 static void
 rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog)
 {
-	GladeXML *xml;
+	GtkWidget  *content_area;
+	GtkBuilder *builder;
 
 	dialog->priv = RB_FEED_PODCAST_PROPERTIES_DIALOG_GET_PRIVATE (dialog);
 
@@ -114,18 +116,16 @@ rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog)
 				 dialog, 0);
 
 	gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 400);
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 
-	xml = rb_glade_xml_new ("podcast-feed-properties.glade",
-				"podcastproperties",
-				dialog);
-	glade_xml_signal_autoconnect (xml);
+	builder = rb_builder_load ("podcast-feed-properties.ui", dialog);
 
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox),
-			   glade_xml_get_widget (xml, "podcastproperties"));
+	gtk_container_add (GTK_CONTAINER (content_area),
+			   GTK_WIDGET (gtk_builder_get_object (builder, "podcastproperties")));
 
 	dialog->priv->close_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
 							    GTK_STOCK_CLOSE,
@@ -133,26 +133,26 @@ rb_feed_podcast_properties_dialog_init (RBFeedPodcastPropertiesDialog *dialog)
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 					 GTK_RESPONSE_CLOSE);
 
-	/* get the widgets from the XML */
-	dialog->priv->title = glade_xml_get_widget (xml, "titleLabel");
-	dialog->priv->author = glade_xml_get_widget (xml, "authorLabel");
-	dialog->priv->location = glade_xml_get_widget (xml, "locationLabel");
-	dialog->priv->language = glade_xml_get_widget (xml, "languageLabel");
-	dialog->priv->last_update = glade_xml_get_widget (xml, "lastupdateLabel");
-	dialog->priv->last_episode = glade_xml_get_widget (xml, "lastepisodeLabel");
-	dialog->priv->copyright = glade_xml_get_widget (xml, "copyrightLabel");
-	dialog->priv->summary = glade_xml_get_widget (xml, "summaryLabel");
+	/* get the widgets from the builder */
+	dialog->priv->title = GTK_WIDGET (gtk_builder_get_object (builder, "titleLabel"));
+	dialog->priv->author = GTK_WIDGET (gtk_builder_get_object (builder, "authorLabel"));
+	dialog->priv->location = GTK_WIDGET (gtk_builder_get_object (builder, "locationLabel"));
+	dialog->priv->language = GTK_WIDGET (gtk_builder_get_object (builder, "languageLabel"));
+	dialog->priv->last_update = GTK_WIDGET (gtk_builder_get_object (builder, "lastupdateLabel"));
+	dialog->priv->last_episode = GTK_WIDGET (gtk_builder_get_object (builder, "lastepisodeLabel"));
+	dialog->priv->copyright = GTK_WIDGET (gtk_builder_get_object (builder, "copyrightLabel"));
+	dialog->priv->summary = GTK_WIDGET (gtk_builder_get_object (builder, "summaryLabel"));
 
-	rb_glade_boldify_label (xml, "titleDescLabel");
-	rb_glade_boldify_label (xml, "authorDescLabel");
-	rb_glade_boldify_label (xml, "locationDescLabel");
-	rb_glade_boldify_label (xml, "languageDescLabel");
-	rb_glade_boldify_label (xml, "lastupdateDescLabel");
-	rb_glade_boldify_label (xml, "lastepisodeDescLabel");
-	rb_glade_boldify_label (xml, "copyrightDescLabel");
-	rb_glade_boldify_label (xml, "summaryDescLabel");
+	rb_builder_boldify_label (builder, "titleDescLabel");
+	rb_builder_boldify_label (builder, "authorDescLabel");
+	rb_builder_boldify_label (builder, "locationDescLabel");
+	rb_builder_boldify_label (builder, "languageDescLabel");
+	rb_builder_boldify_label (builder, "lastupdateDescLabel");
+	rb_builder_boldify_label (builder, "lastepisodeDescLabel");
+	rb_builder_boldify_label (builder, "copyrightDescLabel");
+	rb_builder_boldify_label (builder, "summaryDescLabel");
 
-	g_object_unref (G_OBJECT (xml));
+	g_object_unref (builder);
 }
 
 static void
@@ -263,8 +263,29 @@ static void
 rb_feed_podcast_properties_dialog_update_language (RBFeedPodcastPropertiesDialog *dialog)
 {
 	const char *language;
-
+#if GST_CHECK_VERSION(0,10,26)
+	char *separator;
+	char *iso636lang;
+	const char *langname;
+#endif
 	language = rhythmdb_entry_get_string (dialog->priv->current_entry, RHYTHMDB_PROP_LANG);
+#if GST_CHECK_VERSION(0,10,26)
+	/* language tag is language[-subcode]; we only care about the language bit */
+	iso636lang = g_strdup (language);
+	separator = strchr (iso636lang, '-');
+	if (separator != NULL) {
+		*separator = '\0';
+	}
+
+	/* map the language code to a language name */
+	langname = gst_tag_get_language_name (iso636lang);
+	g_free (iso636lang);
+	if (langname != NULL) {
+		rb_debug ("mapped language code %s to %s", language, langname);
+		gtk_label_set_text (GTK_LABEL (dialog->priv->language), langname);
+		return;
+	}
+#endif
 	gtk_label_set_text (GTK_LABEL (dialog->priv->language), language);
 }
 
