@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of song cut/paste handler object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
@@ -123,6 +121,9 @@ struct RBShellClipboardPrivate
 	GtkActionGroup *actiongroup;
 	guint playlist_menu_ui_id;
 
+	guint delete_action_ui_id;
+	GtkAction *delete_action;
+
 	GHashTable *signal_hash;
 
 	GAsyncQueue *deleted_queue;
@@ -162,33 +163,41 @@ static GtkActionEntry rb_shell_clipboard_actions [] =
 	{ "EditPaste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
 	  N_("Paste selection"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_paste) },
-	{ "EditDelete", GTK_STOCK_REMOVE, N_("_Remove"), NULL,
-	  N_("Remove selection"),
+	{ "EditDelete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
+	  N_("Delete each selected item"),
+	  G_CALLBACK (rb_shell_clipboard_cmd_delete) },
+	{ "EditRemove", GTK_STOCK_REMOVE, N_("_Remove"), NULL,
+	  N_("Remove each selected item from the library"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_delete) },
 	{ "EditMovetoTrash", "user-trash", N_("_Move to Trash"), NULL,
-	  N_("Move selection to the trash"),
+	  N_("Move each selected item to the trash"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_move_to_trash) },
 
 	{ "EditPlaylistAdd", NULL, N_("Add to P_laylist") },
 	{ "EditPlaylistAddNew", RB_STOCK_PLAYLIST_NEW, N_("_New Playlist"), NULL,
-	  N_("Add the selected songs to a new playlist"),
+	  N_("Add each selected song to a new playlist"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_add_to_playlist_new) },
 	{ "AddToQueue", GTK_STOCK_ADD, N_("Add _to Play Queue"), NULL,
-	  N_("Add the selected songs to the play queue"),
+	  N_("Add each selected song to the play queue"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_add_song_to_queue) },
 	{ "QueueDelete", GTK_STOCK_REMOVE, N_("Remove"), NULL,
-	  N_("Remove selection"),
+	  N_("Remove each selected item from the play queue"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_queue_delete) },
 
 	{ "MusicProperties", GTK_STOCK_PROPERTIES, N_("Pr_operties"), "<Alt>Return",
-	  N_("Show information on the selected song"),
+	  N_("Show information on each selected song"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_song_info) },
 	{ "QueueMusicProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
-	  N_("Show information on the selected song"),
+	  N_("Show information on each selected song"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_queue_song_info) },
 };
 static guint rb_shell_clipboard_n_actions = G_N_ELEMENTS (rb_shell_clipboard_actions);
 
+static const char *delete_action_paths[] = {
+	"/MenuBar/EditMenu/DeleteActionPlaceholder",
+	"/BrowserSourceViewPopup/DeleteActionPlaceholder",
+};
+
 static const char *playlist_menu_paths[] = {
 	"/MenuBar/EditMenu/EditPlaylistAddMenu/EditPlaylistAddPlaceholder",
 	"/BrowserSourceViewPopup/BrowserSourcePopupPlaylistAdd/BrowserSourcePopupPlaylistAddPlaceholder",
@@ -280,6 +289,9 @@ unset_source_internal (RBShellClipboard *clipboard)
 							      G_CALLBACK (rb_shell_clipboard_entries_changed_cb),
 							      clipboard);
 		}
+
+		gtk_ui_manager_remove_ui (clipboard->priv->ui_mgr,
+					  clipboard->priv->delete_action_ui_id);
 	}
 	clipboard->priv->source = NULL;
 }
@@ -345,6 +357,7 @@ rb_shell_clipboard_set_source_internal (RBShellClipboard *clipboard,
 
 	if (clipboard->priv->source != NULL) {
 		RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
+		char *delete_action;
 
 		if (songs) {
 			g_signal_connect_object (G_OBJECT (songs),
@@ -364,6 +377,28 @@ rb_shell_clipboard_set_source_internal (RBShellClipboard *clipboard,
 						 G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
 						 clipboard, 0);
 		}
+
+		delete_action = rb_source_get_delete_action (source);
+		if (delete_action != NULL) {
+			char *path;
+			int i;
+			for (i = 0; i < G_N_ELEMENTS (delete_action_paths); i++) {
+				gtk_ui_manager_add_ui (clipboard->priv->ui_mgr,
+						       clipboard->priv->delete_action_ui_id,
+						       delete_action_paths[i],
+						       delete_action,
+						       delete_action,
+						       GTK_UI_MANAGER_AUTO,
+						       FALSE);
+			}
+			gtk_ui_manager_ensure_update (clipboard->priv->ui_mgr);
+
+			/* locate action too */
+			path = g_strdup_printf ("%s/%s", delete_action_paths[0], delete_action);
+			clipboard->priv->delete_action = gtk_ui_manager_get_action (clipboard->priv->ui_mgr, path);
+			g_free (path);
+		}
+		g_free (delete_action);
 	}
 
 	rebuild_playlist_menu (clipboard);
@@ -398,6 +433,9 @@ rb_shell_clipboard_set_property (GObject *object,
 		break;
 	case PROP_UI_MANAGER:
 		clipboard->priv->ui_mgr = g_value_get_object (value);
+		clipboard->priv->delete_action_ui_id =
+			gtk_ui_manager_new_merge_id (clipboard->priv->ui_mgr);
+
 		break;
 	case PROP_PLAYLIST_MANAGER:
 		if (clipboard->priv->playlist_manager != NULL) {
@@ -542,7 +580,7 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	gboolean can_select_all = FALSE;
 	gboolean can_show_properties = FALSE;
 	GtkAction *action;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	if (clipboard->priv->source) {
 		view = rb_source_get_entry_view (clipboard->priv->source);
@@ -576,48 +614,51 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	}
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditCut");
-	g_object_set (G_OBJECT (action), "sensitive", can_cut, NULL);
+	g_object_set (action, "sensitive", can_cut, NULL);
 
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditDelete");
-	g_object_set (G_OBJECT (action), "sensitive", can_delete, NULL);
+	if (clipboard->priv->delete_action != NULL) {
+		g_object_set (clipboard->priv->delete_action, "sensitive", can_delete, NULL);
+	}
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditMovetoTrash");
-	g_object_set (G_OBJECT (action), "sensitive", can_move_to_trash, NULL);
+	g_object_set (action, "sensitive", can_move_to_trash, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditCopy");
-	g_object_set (G_OBJECT (action), "sensitive", can_copy, NULL);
+	g_object_set (action, "sensitive", can_copy, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup,"EditPaste");
-	g_object_set (G_OBJECT (action), "sensitive", can_paste, NULL);
+	g_object_set (action, "sensitive", can_paste, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditPlaylistAdd");
-	g_object_set (G_OBJECT (action), "sensitive", can_copy, NULL);
+	g_object_set (action, "sensitive", can_copy, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "AddToQueue");
-	g_object_set (G_OBJECT (action), "sensitive", can_add_to_queue, NULL);
+	g_object_set (action, "sensitive", can_add_to_queue, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "MusicProperties");
-	g_object_set (G_OBJECT (action), "sensitive", can_show_properties, NULL);
+	g_object_set (action, "sensitive", can_show_properties, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueMusicProperties");
-	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
+	g_object_set (action, "sensitive", have_sidebar_selection, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueDelete");
-	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
+	g_object_set (action, "sensitive", have_sidebar_selection, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditSelectAll");
-	g_object_set (G_OBJECT (action), "sensitive", can_select_all, NULL);
+	g_object_set (action, "sensitive", can_select_all, NULL);
 
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditSelectNone");
-	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
+	g_object_set (action, "sensitive", have_selection, NULL);
 
 	/* disable the whole add-to-playlist menu if the source's entry type doesn't have playlists */
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditPlaylistAdd");
 	if (clipboard->priv->source != NULL) {
 		g_object_get (clipboard->priv->source, "entry-type", &entry_type, NULL);
-		if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID) {
-			gtk_action_set_sensitive (action, entry_type->has_playlists);
-			g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+		if (entry_type != NULL) {
+			gboolean has_playlists;
+			g_object_get (entry_type, "has-playlists", &has_playlists, NULL);
+			gtk_action_set_sensitive (action, has_playlists);
+			g_object_unref (entry_type);
 		} else {
 			gtk_action_set_sensitive (action, FALSE);
 		}
@@ -653,7 +694,9 @@ rb_shell_clipboard_cmd_select_all (GtkAction *action,
 	} else {
 		/* select all tracks in the entry view */
 		entryview = rb_source_get_entry_view (clipboard->priv->source);
-		rb_entry_view_select_all (entryview);
+		if (entryview != NULL) {
+			rb_entry_view_select_all (entryview);
+		}
 	}
 }
 
@@ -670,7 +713,9 @@ rb_shell_clipboard_cmd_select_none (GtkAction *action,
 		gtk_editable_select_region (GTK_EDITABLE (widget), -1, -1);
 	} else {
 		entryview = rb_source_get_entry_view (clipboard->priv->source);
-		rb_entry_view_select_none (entryview);
+		if (entryview != NULL) {
+			rb_entry_view_select_none (entryview);
+		}
 	}
 }
 
@@ -906,8 +951,8 @@ add_playlist_to_menu (GtkTreeModel *model,
 		      GtkTreeIter *iter,
 		      RBShellClipboard *clipboard)
 {
-	RhythmDBEntryType entry_type;
-	RhythmDBEntryType source_entry_type;
+	RhythmDBEntryType *entry_type;
+	RhythmDBEntryType *source_entry_type;
 	RBSource *source = NULL;
 	char *action_name;
 	GtkAction *action;
@@ -933,8 +978,8 @@ add_playlist_to_menu (GtkTreeModel *model,
 	g_object_get (source, "entry-type", &source_entry_type, NULL);
 	if (source_entry_type != entry_type) {
 		g_object_unref (source);
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+		g_object_unref (entry_type);
+		g_object_unref (source_entry_type);
 		return FALSE;
 	}
 
@@ -971,8 +1016,8 @@ add_playlist_to_menu (GtkTreeModel *model,
 				       GTK_UI_MANAGER_AUTO, FALSE);
 	}
 
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, source_entry_type);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (source_entry_type);
+	g_object_unref (entry_type);
 	g_free (action_name);
 	g_object_unref (source);
 
