@@ -42,18 +42,20 @@ G_BEGIN_DECLS
 #define RB_IS_SHELL_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_SHELL))
 #define RB_SHELL_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_SHELL, RBShellClass))
 
-enum
+typedef enum
 {
 	RB_SHELL_ERROR_NO_SUCH_URI,
 	RB_SHELL_ERROR_NO_SUCH_PROPERTY,
 	RB_SHELL_ERROR_IMMUTABLE_PROPERTY,
 	RB_SHELL_ERROR_INVALID_PROPERTY_TYPE,
 	RB_SHELL_ERROR_NO_SOURCE_FOR_URI
-};
+} RBShellError;
 
 #define RB_SHELL_ERROR rb_shell_error_quark ()
 
 GQuark rb_shell_error_quark (void);
+GType rb_shell_error_get_type (void);
+#define RB_TYPE_SHELL_ERROR (rb_shell_error_get_type ())
 
 typedef enum
 {
@@ -87,6 +89,7 @@ struct _RBShellClass
 	void	 (*visibility_changed)	(RBShell *shell, gboolean visible);
 	void	 (*create_song_info)	(RBShell *shell, RBSongInfo *song_info, gboolean multi);
 	void	 (*removable_media_scan_finished) (RBShell *shell);
+	void     (*database_load_complete) (RBShell *shell);
 };
 
 GType		rb_shell_get_type	(void);
@@ -143,6 +146,11 @@ gboolean	rb_shell_clear_queue (RBShell *shell,
 gboolean	rb_shell_quit (RBShell *shell,
 			       GError **error);
 
+gboolean	rb_shell_activate_source_by_uri (RBShell *shell,
+						 const char *source_uri,
+						 guint play,
+						 GError **error);
+
 void            rb_shell_notify_custom  (RBShell *shell,
 					 guint timeout,
 					 const char *primary,
@@ -155,9 +163,9 @@ gboolean	rb_shell_do_notify (RBShell *shell,
 
 void            rb_shell_register_entry_type_for_source (RBShell *shell,
 							 RBSource *source,
-							 RhythmDBEntryType type);
+							 RhythmDBEntryType *type);
 RBSource * rb_shell_get_source_by_entry_type (RBShell *shell,
-					      RhythmDBEntryType type);
+					      RhythmDBEntryType *type);
 
 gboolean        rb_shell_get_party_mode (RBShell *shell);
 
