@@ -1,6 +1,4 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
- *
- *  arch-tag: Implementation of status display widget
  *
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
@@ -10,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -31,13 +37,29 @@
 #include <gtk/gtk.h>
 
 #include "rb-statusbar.h"
+#include "rb-track-transfer-queue.h"
 #include "rb-debug.h"
 
+/**
+ * SECTION:rb-statusbar
+ * @short_description: status bar widget
+ *
+ * The status bar is displayed at the bottom of the main window.  It consists of some
+ * status text and a progress bar.
+ *
+ * The status text usually comes from the selected source, and typically shows the number
+ * of songs, the total duration and the total file size.  When a menu is open, however, 
+ * the status text shows the description of the currently selected menu item.
+ *
+ * The progress bar shows progress information from a variety of sources.  The source that
+ * is currently selected in the source list can provide progress information, such as buffering
+ * feedback, track transfer status, or progress for updating a song catalog.  If the source
+ * does not provide status information and the database is busy (loading the database from disk,
+ * processing a query, etc.) the progress bar will be pulsed periodically.
+ */
+
 #define EPSILON		(0.00001)
 
-static GObject* rb_statusbar_construct (GType type,
-					guint n_construct_properties,
-					GObjectConstructParam *construct_properties);
 static void rb_statusbar_class_init (RBStatusbarClass *klass);
 static void rb_statusbar_init (RBStatusbar *statusbar);
 static void rb_statusbar_dispose (GObject *object);
@@ -55,22 +77,23 @@ static gboolean poll_status (RBStatusbar *status);
 static void rb_statusbar_sync_status (RBStatusbar *status);
 static void rb_statusbar_source_status_changed_cb (RBSource *source,
 						   RBStatusbar *statusbar);
+static void rb_statusbar_transfer_progress_cb (RBTrackTransferQueue *queue,
+					       int done,
+					       int total,
+					       double fraction,
+					       int time_left,
+					       RBStatusbar *statusbar);
 
 struct RBStatusbarPrivate
 {
         RBSource *selected_source;
+	RBTrackTransferQueue *transfer_queue;
 
         RhythmDB *db;
 
         GtkUIManager *ui_manager;
-        GtkTooltips *tooltips;
 
         GtkWidget *progress;
-        double progress_fraction;
-        gboolean progress_changed;
-        gchar *progress_text;
-
-        gchar *loading_text;
 
         guint status_poll_id;
 };
@@ -80,7 +103,8 @@ enum
         PROP_0,
         PROP_DB,
         PROP_UI_MANAGER,
-        PROP_SOURCE
+        PROP_SOURCE,
+	PROP_TRANSFER_QUEUE
 };
 
 G_DEFINE_TYPE (RBStatusbar, rb_statusbar, GTK_TYPE_STATUSBAR)
@@ -90,13 +114,17 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->constructor = rb_statusbar_construct;
         object_class->dispose = rb_statusbar_dispose;
         object_class->finalize = rb_statusbar_finalize;
 
         object_class->set_property = rb_statusbar_set_property;
         object_class->get_property = rb_statusbar_get_property;
 
+	/**
+	 * RBStatusbar:db:
+	 *
+	 * The #RhythmDB instance
+	 */
         g_object_class_install_property (object_class,
                                          PROP_DB,
                                          g_param_spec_object ("db",
@@ -104,6 +132,11 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               "RhythmDB object",
                                                               RHYTHMDB_TYPE,
                                                               G_PARAM_READWRITE));
+	/**
+	 * RBStatusbar:source:
+	 *
+	 * The currently selected #RBSource
+	 */
         g_object_class_install_property (object_class,
                                          PROP_SOURCE,
                                          g_param_spec_object ("source",
@@ -111,6 +144,11 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               "RBSource object",
                                                               RB_TYPE_SOURCE,
                                                               G_PARAM_READWRITE));
+	/**
+	 * RBStatusbar:ui-manager:
+	 *
+	 * The #GtkUIManager instance
+	 */
         g_object_class_install_property (object_class,
                                          PROP_UI_MANAGER,
                                          g_param_spec_object ("ui-manager",
@@ -119,29 +157,22 @@ rb_statusbar_class_init (RBStatusbarClass *klass)
                                                               GTK_TYPE_UI_MANAGER,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * RBStatusbar::transfer-queue:
+	 *
+	 * The #RBTrackTransferQueue instance
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_TRANSFER_QUEUE,
+					 g_param_spec_object ("transfer-queue",
+							      "RBTrackTransferQueue",
+							      "RBTrackTransferQueue instance",
+							      RB_TYPE_TRACK_TRANSFER_QUEUE,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	g_type_class_add_private (klass, sizeof (RBStatusbarPrivate));
 }
 
-static GObject*
-rb_statusbar_construct (GType                  type,
-                        guint                  n_construct_properties,
-                        GObjectConstructParam *construct_properties)
-{
-        RBStatusbarClass *klass;
-        GObject *object;
-        RBStatusbar *statusbar;
-
-        klass = RB_STATUSBAR_CLASS (g_type_class_peek (RB_TYPE_STATUSBAR));
-        object = G_OBJECT_CLASS (rb_statusbar_parent_class)->constructor
-					(type,
-					 n_construct_properties,
-					 construct_properties);
-
-        statusbar = RB_STATUSBAR (object);
-
-        return object;
-}
-
 static void
 rb_statusbar_init (RBStatusbar *statusbar)
 {
@@ -149,15 +180,10 @@ rb_statusbar_init (RBStatusbar *statusbar)
 						       RB_TYPE_STATUSBAR,
 						       RBStatusbarPrivate);
 
-        statusbar->priv->tooltips = gtk_tooltips_new ();
-        gtk_tooltips_enable (statusbar->priv->tooltips);
-
         gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), TRUE);
 
         statusbar->priv->progress = gtk_progress_bar_new ();
-        statusbar->priv->progress_fraction = 1.0;
-
-        statusbar->priv->loading_text = g_strdup (_("Loading..."));
+	gtk_widget_set_size_request (statusbar->priv->progress, -1, 10);
 
         gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (statusbar->priv->progress), 1.0);
         gtk_widget_hide (statusbar->priv->progress);
@@ -198,6 +224,11 @@ rb_statusbar_dispose (GObject *object)
 		statusbar->priv->selected_source = NULL;
 	}
 
+	if (statusbar->priv->transfer_queue != NULL) {
+		g_object_unref (statusbar->priv->transfer_queue);
+		statusbar->priv->transfer_queue = NULL;
+	}
+
         G_OBJECT_CLASS (rb_statusbar_parent_class)->dispose (object);
 }
 
@@ -213,9 +244,6 @@ rb_statusbar_finalize (GObject *object)
 
         g_return_if_fail (statusbar->priv != NULL);
 
-	g_free (statusbar->priv->loading_text);
-	g_free (statusbar->priv->progress_text);
-
         G_OBJECT_CLASS (rb_statusbar_parent_class)->finalize (object);
 }
 
@@ -338,6 +366,14 @@ rb_statusbar_set_property (GObject *object,
                                          statusbar,
                                          G_CONNECT_SWAPPED);
                 break;
+	case PROP_TRANSFER_QUEUE:
+		statusbar->priv->transfer_queue = g_value_dup_object (value);
+		g_signal_connect_object (G_OBJECT (statusbar->priv->transfer_queue),
+					 "transfer-progress",
+					 G_CALLBACK (rb_statusbar_transfer_progress_cb),
+					 statusbar,
+					 0);
+		break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -363,15 +399,24 @@ rb_statusbar_get_property (GObject *object,
         case PROP_UI_MANAGER:
                 g_value_set_object (value, statusbar->priv->ui_manager);
                 break;
+        case PROP_TRANSFER_QUEUE:
+                g_value_set_object (value, statusbar->priv->transfer_queue);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
         }
 }
 
+/**
+ * rb_statusbar_set_source:
+ * @statusbar: the #RBStatusbar
+ * @source: the new selected #RBSource
+ *
+ * Updates the status bar for a newly selected source.
+ */
 void
-rb_statusbar_set_source (RBStatusbar *statusbar,
-                            RBSource *source)
+rb_statusbar_set_source (RBStatusbar *statusbar, RBSource *source)
 {
         g_return_if_fail (RB_IS_STATUSBAR (statusbar));
         g_return_if_fail (RB_IS_SOURCE (source));
@@ -398,24 +443,25 @@ static void
 rb_statusbar_sync_status (RBStatusbar *status)
 {
         gboolean changed = FALSE;
-        gboolean show_progress = TRUE;
         char *status_text = NULL;
 	char *progress_text = NULL;
 	float progress = 999;
+	int time_left = 0;
 
         /*
          * Behaviour of status bar:
 	 * - use source's status text
-	 * - use source's progress value and text, unless internal progress value is set,
-	 * - if no source progress value or internal progress value and library is busy,
+	 * - use source's progress value and text, unless transfer queue provides something
+	 * - if no source progress value or transfer progress value and library is busy,
 	 *    pulse the progress bar
          */
         
 	/* library busy? */
         if (rhythmdb_is_busy (status->priv->db)) {
-		/*g_free (status_text);
-		status_text = g_strdup (status->priv->loading_text); */
 		progress = -1.0f;
+
+		/* see if it wants to provide more details */
+		rhythmdb_get_progress_info (status->priv->db, &progress_text, &progress);
 		changed = TRUE;
         }
 
@@ -426,16 +472,12 @@ rb_statusbar_sync_status (RBStatusbar *status)
 			status_text ? status_text : "", progress_text ? progress_text : "", progress);
 	}
 
-        /* internal progress bar moving? */
-        if (status->priv->progress_fraction < (1.0 - EPSILON) || status->priv->progress_changed) {
-		g_free (progress_text);
-                progress_text = g_strdup (status->priv->progress_text);
-		progress = status->priv->progress_fraction;
-
-                status->priv->progress_changed = FALSE;
-		show_progress = TRUE;
-		changed = TRUE;
-        }
+	/* get transfer details */
+	rb_track_transfer_queue_get_status (status->priv->transfer_queue,
+					    &status_text,
+					    &progress_text,
+					    &progress,
+					    &time_left);
 
         /* set up the status text */
         if (status_text) {
@@ -445,12 +487,12 @@ rb_statusbar_sync_status (RBStatusbar *status)
         }
 
         /* set up the progress bar */
-	if (progress > (1.0f - EPSILON) || !show_progress) {
+	if (progress > (1.0f - EPSILON)) {
 		gtk_widget_hide (status->priv->progress);
 	} else {
                 gtk_widget_show (status->priv->progress);
 
-                if (progress < (0.0f - EPSILON)) {
+                if (progress < EPSILON) {
 			gtk_progress_bar_pulse (GTK_PROGRESS_BAR (status->priv->progress));
                         changed = TRUE;
                 } else {
@@ -467,13 +509,25 @@ rb_statusbar_sync_status (RBStatusbar *status)
                 status->priv->status_poll_id = g_timeout_add (250, (GSourceFunc) poll_status, status);
 }
 
+/**
+ * rb_statusbar_new:
+ * @db: the #RhythmDB instance
+ * @ui_manager: the #GtkUIManager
+ * @transfer_queue: the #RBTrackTransferQueue
+ *
+ * Creates the status bar widget.
+ *
+ * Return value: the status bar widget
+ */
 RBStatusbar *
 rb_statusbar_new (RhythmDB *db,
-                  GtkUIManager *ui_manager)
+                  GtkUIManager *ui_manager,
+		  RBTrackTransferQueue *queue)
 {
         RBStatusbar *statusbar = g_object_new (RB_TYPE_STATUSBAR,
                                                "db", db,
                                                "ui-manager", ui_manager,
+					       "transfer-queue", queue,
                                                NULL);
 
         g_return_val_if_fail (statusbar->priv != NULL, NULL);
@@ -481,32 +535,29 @@ rb_statusbar_new (RhythmDB *db,
         return statusbar;
 }
 
-void
-rb_statusbar_set_progress (RBStatusbar *statusbar, double progress, const char *text)
+static void
+add_status_poll (RBStatusbar *statusbar)
 {
-        if (statusbar->priv->progress_text) {
-                g_free (statusbar->priv->progress_text);
-                statusbar->priv->progress_text = NULL;
-        }
-
-        if (progress > (0.0 - EPSILON)) {
-                statusbar->priv->progress_fraction = progress;
-                statusbar->priv->progress_changed = TRUE;
-		if (text)
-			statusbar->priv->progress_text = g_strdup (text);
-        } else {
-                /* trick sync_status into hiding it */
-                statusbar->priv->progress_fraction = 1.0;
-                statusbar->priv->progress_changed = FALSE;
-        }
-        rb_statusbar_sync_status (statusbar);
+        if (statusbar->priv->status_poll_id == 0)
+                statusbar->priv->status_poll_id =
+                        g_idle_add ((GSourceFunc) poll_status, statusbar);
 }
 
 static void
 rb_statusbar_source_status_changed_cb (RBSource *source, RBStatusbar *statusbar)
 {
 	rb_debug ("source status changed");
-        if (statusbar->priv->status_poll_id == 0)
-                statusbar->priv->status_poll_id =
-                        g_idle_add ((GSourceFunc) poll_status, statusbar);
+	add_status_poll (statusbar);
+}
+
+static void
+rb_statusbar_transfer_progress_cb (RBTrackTransferQueue *queue,
+				   int done,
+				   int total,
+				   double progress,
+				   int time_left,
+				   RBStatusbar *statusbar)
+{
+	rb_debug ("transfer progress changed");
+	add_status_poll (statusbar);
 }
