@@ -26,7 +26,7 @@
  *
  */
 
-/**
+/*
  * SECTION:rb-podcast-source
  * @short_description: source displaying podcast feeds and episodes
  *
@@ -47,6 +47,7 @@
 #include <gtk/gtk.h>
 
 #include "rb-podcast-source.h"
+#include "rb-podcast-entry-types.h"
 
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
@@ -254,7 +255,13 @@ static void impl_get_status				(RBSource *source,
 							 char **progress_text,
 							 float *progress);
 static guint impl_want_uri				(RBSource *source, const char *uri);
-static gboolean impl_add_uri				(RBSource *source, const char *uri, const char *title, const char *genre);
+static gboolean impl_add_uri				(RBSource *source,
+							 const char *uri,
+							 const char *title,
+							 const char *genre,
+							 RBSourceAddCallback callback,
+							 gpointer data,
+							 GDestroyNotify destroy_data);
 static char *impl_get_delete_action			(RBSource *source);
 
 #define CMD_PATH_SHOW_BROWSER "/commands/ShowBrowser"
@@ -817,7 +824,7 @@ rb_podcast_source_get_property (GObject *object,
 	}
 }
 
-/**
+/*
  * rb_podcast_source_new:
  * @shell: the #RBShell instance
  *
@@ -1127,7 +1134,7 @@ static GPtrArray *
 construct_query_from_selection (RBPodcastSource *source)
 {
 	GPtrArray *query;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	g_object_get (source, "entry-type", &entry_type, NULL);
 
@@ -1136,7 +1143,7 @@ construct_query_from_selection (RBPodcastSource *source)
 				      RHYTHMDB_PROP_TYPE,
 				      entry_type,
 				      RHYTHMDB_QUERY_END);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	if (source->priv->search_query) {
 		rhythmdb_query_append (source->priv->db,
@@ -1270,7 +1277,7 @@ posts_view_drag_data_received_cb (GtkWidget *widget,
 	impl_receive_drag (RB_SOURCE (source), selection_data);
 }
 
-/**
+/*
  * rb_podcast_source_add_feed:
  * @source: the #RBPodcastSource
  * @uri: the new feed to add
@@ -2065,10 +2072,18 @@ impl_want_uri (RBSource *source, const char *uri)
 }
 
 static gboolean
-impl_add_uri (RBSource *asource, const char *uri, const char *title, const char *genre)
+impl_add_uri (RBSource *asource,
+	      const char *uri,
+	      const char *title,
+	      const char *genre,
+	      RBSourceAddCallback callback,
+	      gpointer data,
+	      GDestroyNotify destroy_data)
 {
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
 	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, uri, FALSE);
+	callback (asource, uri, data);
+	destroy_data (data);
 	return TRUE;
 }
 
@@ -2078,7 +2093,7 @@ rb_podcast_source_entry_changed_cb (RhythmDB *db,
 				    GValueArray *changes,
 				    RBPodcastSource *source)
 {
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	gboolean feed_changed;
 	int i;
 
