@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header for Internet Radio source object
- *
  *  Copyright (C) 2005 Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,8 +28,8 @@
 #ifndef __RB_PODCAST_SOURCE_H
 #define __RB_PODCAST_SOURCE_H
 
-#include "rb-shell.h"
-#include "rb-source.h"
+#include <shell/rb-shell.h>
+#include <sources/rb-source.h>
 
 G_BEGIN_DECLS
 
